import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits import mplot3d
import math



jump_duration = 9
jump_height = 5
step_size = 1.5
map_limit = 5


# represents a "fake player", an object that moves around in 3D space semi-randomly

class MovingObject:
    """
    This generator creates random continous paths for n-players in n-given timesteps.
    
    This class is used to simulate a path for training the ML model. 
    
    Note: This algorithm introduces only randomization and is not sutible for ML.

    Example usage:

    mov = MovingObject()

    x, y, z, c = mov.generate(5, 500)

    """

    def __init__(self, pos=np.zeros(3)):
        self.pos = np.array(pos)
        self.direction = np.random.uniform(0.0, math.pi * 2)
        self.lr = 1
        self.jump = 0
        self.stairs = 0
        self.ascstair = True

    def step(self):
        ascent = 0
        if self.jump == -1:
            self.direction += self.lr * np.random.uniform(0, 0.1)
            if np.random.random_sample() < 0.01: self.lr *= -1
            if np.random.random_sample() < 0.03: self.jump = jump_duration
            if np.random.random_sample() < 0.03:
                self.stairs = np.random.randint(8, 16)
                self.ascstair = True if np.random.random_sample() < 0.5 else False
        else:
            ascent = (self.jump - (jump_duration / 2)) / 1.2
            self.jump -= 1

        if self.stairs > 0 and ascent == 0:
            ascent = 0.8 if self.ascstair else -0.8
            self.stairs -= 1


        midpos = self.pos + [step_size * math.cos(self.direction), step_size * math.sin(self.direction), ascent]
        if -map_limit < midpos[0] < map_limit and -map_limit < midpos[1] < map_limit and -5 < midpos[2] < 5:
            self.pos += [step_size * math.cos(self.direction), step_size * math.sin(self.direction), ascent]
            #self.direction = -1*self.direction
            #self.step()
            #print(self.pos)

    def generate(self, n_players, timesteps):
        mobs = [MovingObject() for _ in range(n_players)]

        xdata, ydata, zdata, cdata = (
            [-100, 100], [-100, 100], [-100, 100],
            [0, 0])  # some initial points to expand the boundaries and prevent weird scaling
        color = 0
        for mob in mobs:
            color += 1

            for _ in range(timesteps):
                mob.step()
                xdata.append(mob.pos[0])
                ydata.append(mob.pos[1])
                zdata.append(mob.pos[2])
                cdata.append(color)
        return xdata, ydata, zdata, cdata

    def show_plot(self):
        ax = plt.axes(projection='3d')
        xdata, ydata, zdata, cdata = self.generate(5, 500)
        ax.scatter3D(xdata, ydata, zdata, c=cdata)
        plt.show()

if __name__ == "__main__":
    print("Should be called in a class and not be run in standalone.")