import numpy as np
import pandas as pd
import os
import re

#gets the data for one player from file
#specify the indexes of boolean features, so they can be rewritten to numbers
def get_player(file, boolean_indexes):
    player = np.array(pd.read_csv(file, delimiter=",", header=None))
    for row in player:
        for index in boolean_indexes:
            if row[index]:
                row[index] = 1.0
            else:
                row[index] = 0.0
    return player


#gets the data for a given match from files corresponding to each player in the match
#specify the indexes of boolean features, so they can be rewritten to numbers
def get_match(files, boolean_indexes):
    min_length = 10000000
    players = []
    for file in files:
        player = get_player(file, boolean_indexes)
        if len(player) < min_length:
            min_length = len(player)
        players.append(player)
    for j in range (len(players)):
        if len(players[j]) <= min_length:
            continue
        players[j] = players[j][:min_length]

    array = np.concatenate((players[0], players[1]), axis=1)
    for i in range(2, len(players)):
        array = np.concatenate((array, players[i]), axis=1)

    return array


#Returns all the files from a given directory in an array [matches, files]
#All timesteps after the first player reaches the goal is removed
def get_files(rootdir, n_players):
    matches = []
    all_files = []
    for subdir, dirs, files in os.walk(rootdir):
        files = sorted(files)
        for file in files:
            match = re.search("MatchData[0-9]*", file) #match.*_(normal|nosetback)
            if not match:
                continue
            if match.group(0) in matches:
                continue
            match = match.group(0)
            matches.append(match)
        for i in range(len(matches)):
            match_files = []
            for j in range(n_players):
                path = os.path.join(subdir, files[(i*n_players)+j])
                match_files.append(path)
            all_files.append(match_files)
    return matches, all_files


#Rewrites a dataset from each player in a match having their own file, to each file containing all the data for a match
def rewrite(rootdir, n_players, boolean_indexes):
    matches, files = get_files(rootdir, n_players)
    path = "Data/Datasets/larger_set"
    os.mkdir(path)
    for i in range(len(matches)):
        new_file = path + "/" + matches[i] + ".csv"
        data = get_match(files[i], boolean_indexes)

        f = open(new_file, "w+")
        for row in data:
            for j in range(len(row)):
                if j == (len(row)-1):
                    f.write(str(row[j]))
                else:
                    f.write(str(row[j]) + ",")
            f.write("\n")
        f.close()


#Groups n_matches to represent a single match
#Was used for testing how the models handle a larger amount of players
def group_matches(n_matches, rootdir, new_file):

    matches = []
    min_length = 1000000
    for subdir, dirs, files in os.walk(rootdir):
        for file in sorted(files):
            print(file)
            path = "Data/Datasets/set1/" + file
            match = np.array(pd.read_csv(path, delimiter=",", header=None))
            matches.append(match)
            if match.shape[0] < min_length:
                min_length = match.shape[0]
            print(match.shape[0])
    for i in range(len(matches)):
        matches[i] = matches[i][:min_length]


    array = np.concatenate((matches[0], matches[1]), axis=1)
    print(array.shape)
    for i in range(2, n_matches):
        array = np.concatenate((array, matches[i]), axis=1)

    print(array.shape)

    f = open(new_file, "w+")
    for row in array:
        for j in range(len(row)):
            if j == (len(row) - 1):
                f.write(str(row[j]))
            else:
                f.write(str(row[j]) + ",")
        f.write("\n")
    f.close()






if __name__ == "__main__":
    indexes = [3, 4]
    rewrite("/home/petter/Bachelor/clean_samples", 4, indexes)