import numpy as np
import sys
from scipy import sparse as sp


class MapGenerator:
    """
    Helper class for creation of empty matrices that represent a given map with a given density.
    """

    def __init__(self, x, y, z, density):
        self.length = x
        self.width = y
        self.height = z
        self.density = density
    
    def create_empty_map_4d(self, initial_value, n_players=3):
        """
        Creates a map in 4D. This map consists of x, y, z and n_players.

        :param inital_value: what value each element in matrix should be.
        :param n_players: how many players that the map will be made for. Default n_players = 3

        :return: empty map

        """
        empty_map = np.full((self.length * self.density, self.width * self.density, self.height * self.density, n_players), initial_value)
        return empty_map

    def create_empty_map_3d(self, initial_value, n_players=3):
        """
        Creates a map in 3D. This map consists of x, y and z.

        :param inital_value: what value each element in matrix should be.
        :param n_players: how many players that the map will be made for. Default n_players = 3

        :return: empty map

        """
        empty_map = np.full((self.length * self.density, self.width * self.density, self.height * self.density), initial_value)
        return empty_map

if __name__ == "__main__":
    print("Should be called in a class and not be run in standalone.")