import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
import Data.DataFormatter as df
import numpy as np
from sklearn import preprocessing

import Visualize.Visualize as vis
from datetime import datetime
import Data.InterestPoint as ip


"""
Tries to predict interest points directly with carthesian coordinates
Uses the alteternative algorithm for interesting positions
Not mentioned in the paper
"""

class InterestModel (nn.Module):

    def __init__(self, input_size, hidden_size, device):
        super(InterestModel, self).__init__()
        self.hidden_size = hidden_size
        self.device = device

        self.lstm = nn.LSTM(input_size, hidden_size)
        self.hidden = self.init_hidden()
        self.leakyRelu = torch.nn.LeakyReLU()
        self.linear = torch.nn.Linear(input_size, 4)

    def init_hidden(self):
        return torch.randn(1, 1, self.hidden_size, device=self.device), torch.randn(1, 1, self.hidden_size, device=self.device)


    def forward(self, map):
        lstm_out, self.hidden = self.lstm(map.view(1, 1, -1), self.hidden)
        #lstm_out = F.tanh(lstm_out[0][0])
        lstm_out = self.leakyRelu(lstm_out)
        lstm_out = self.linear(lstm_out)
        return lstm_out


#Gets the data from directory scaled to [0, 1], and gets the corresponding most interesting positions
#reshapes the numpy arrays for each match to [timesteps, features] and converts to tensors
def get_data():
    x_train, x_test = df.get_data_from_dir("Data/Datasets/set1", 2, scale=True)
    int_point = ip.InterestPoint([0.98, 0.50, 0.82])
    y_train = int_point.get_most_interesting_positions_alt(x_train, 20)
    y_test = int_point.get_most_interesting_positions_alt(x_test, 20)

    for i in range(len(x_train)):
        x_train[i] = np.reshape(x_train[i], [x_train[i].shape[0], -1])
        x_train[i] = torch.from_numpy(x_train[i]).float()
        y_train[i] = np.reshape(y_train[i], [y_train[i].shape[0], -1])
        y_train[i] = torch.from_numpy(y_train[i]).float()

    for i in range(len(x_test)):
        x_test[i] = np.reshape(x_test[i], [x_test[i].shape[0], -1])
        x_test[i] = torch.from_numpy(x_test[i]).float()
        y_test[i] = np.reshape(y_test[i], [y_test[i].shape[0], -1])
        y_test[i] = torch.from_numpy(y_test[i]).float()

    return x_train, y_train, x_test, y_test


#The main training loop
def run(step_size):
    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
    x_train, y_train, x_test, y_test = get_data()
    model = InterestModel(x_train[0].shape[1], x_train[0].shape[1], device)
    model.to(device)
    date = datetime.now()

    loss_func = torch.nn.MSELoss()
    optimizer = optim.Adam(model.parameters(), lr=0.0001)

    lowest_loss = 10000.0
    for epoch in range(5000):
        print("epoch %i" % epoch)
        print(datetime.now().time())
        running_loss = 0.0
        for j in range(len(x_train)):
            data_in = x_train[j].to(device)
            data_out = y_train[j].to(device)
            #print("map")

            for i in range(len(data_in)-step_size):
                optimizer.zero_grad()
                model.hidden = model.init_hidden()

                map_in = data_in[i]
                target = data_out[i+step_size]

                output = model(map_in)
                loss = loss_func(output, target)
                running_loss += loss

                loss.backward()
                optimizer.step()

        print("loss: %f" % running_loss)
        if running_loss < lowest_loss:
            lowest_loss = running_loss
            torch.save(model.state_dict(), "Data/Models/intv2/leaky_linear_ss1_5k_" + str(date) + ".pt")
            print("saved")


if __name__ == "__main__":
    n_players = 2
    step_size = 1
    run(step_size)

