
import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
import Data.DataFormatter as df
import numpy as np
from sklearn import preprocessing

import Visualize.Visualize as vis
from datetime import datetime

"""
Tries to predict future positions of players with carthesian coordinates
Not mentioned in the paper
"""


class PositionModel (nn.Module):
    def __init__(self, input_size, hidden_size, device):
        super(PositionModel, self).__init__()
        self.hidden_size = hidden_size
        self.device = device

        self.lstm = nn.LSTM(input_size, hidden_size)
        self.hidden = self.init_hidden()
        self.leakyRelu = torch.nn.LeakyReLU()
        self.linear = torch.nn.Linear(hidden_size, input_size)

    def init_hidden(self):
        return torch.randn(1, 1, self.hidden_size, device=self.device), torch.randn(1, 1, self.hidden_size, device=self.device)


    def forward(self, map):
        lstm_out, self.hidden = self.lstm(map.view(1, 1, -1), self.hidden)
        lstm_out = F.tanh(lstm_out[0][0])
        lstm_out = self.leakyRelu(lstm_out)
        lstm_out = self.linear(lstm_out)
        return lstm_out

#Gets the data from directory, reshapes to [timesteps, features] for each match, and converts to tensors
def get_data():
    train, test = df.get_data_from_dir("Data/Datasets/set1", 2, scale=True)

    for i in range(len(train)):
        train[i] = np.reshape(train[i], [train[i].shape[0], -1])
        train[i] = torch.from_numpy(train[i]).float()

        if i < len(test):
            test[i] = np.reshape(test[i], [test[i].shape[0], -1])
            test[i] = torch.from_numpy(test[i]).float()


    return train, test


#The main training loop
def run(step_size, hidden_size):
    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
    train, test = get_data()
    model = PositionModel(train[0].shape[1], hidden_size, device)
    model.to(device)
    date = datetime.now()

    loss_func = torch.nn.MSELoss()
    optimizer = optim.Adam(model.parameters(), lr=0.0001)

    lowest_loss = 10000.0
    for epoch in range(10):
        print("epoch %i" % epoch)
        print(datetime.now().time())
        running_loss = 0.0
        for data in train:
            data = data.to(device)
            #print("map")

            for i in range(len(data)-step_size):
                optimizer.zero_grad()
                model.hidden = model.init_hidden()

                map_in = data[i]
                target = data[i+step_size]

                output = model(map_in)
                loss = loss_func(output, target)
                running_loss += loss

                loss.backward()
                optimizer.step()

        print("loss: %f" % running_loss)
        if running_loss < lowest_loss:
            lowest_loss = running_loss
            torch.save(model.state_dict(), "Data/Models/PositionModel/test_hidden_" + str(date) + ".pt")
            print("saved")


def evaluate(predicted, target):
    loss = (np.square(target - predicted)).mean(axis=None)
    return loss


if __name__ == "__main__":
    n_players = 2
    step_size = 20
    hidden_size = 64
    run(step_size, hidden_size)

