import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import Data.DataFormatter as df
import Data.InterestPoint as ip
import Visualize.Visualize as vis
import numpy as np
import torch

import Predictors.Utility as util
import Models.ConvModel as cm



#Loads the trained model with the weights specified by path
#device is a string that should be "cuda:0" for gpu and "cpu" for cpu
def load_model(factor, path, device):
    model = cm.ConvModel(factor)
    model.load_state_dict(torch.load(path, map_location=device))
    model.eval()
    return model


#Returns predictions and their respective targets for the given match, specified by map_nr
#shape is the shape of the map, and factor is the factor it should be scaled by
def predict(data, factor, model, shape, map_nr):
    with torch.no_grad():
        int_point = ip.InterestPoint([1980, 470, 210])
        y = int_point.get_most_interesting_positions(data)
        data = data[map_nr]
        predictions = np.empty([data.shape[map_nr], 20, 10])
        for i in range(data.shape[0]):
            if i % 100 == 0:
                print(i, " av ", data.shape[map_nr])
            grid = to_grid_2d(data[i], factor, shape)

            pred = model(grid)
            pred = pred.view(20, 10)
            predictions[i] = pred.cpu().numpy()
    return predictions, y


#creates a grid with the size of shape scaled by factor, and initializes all elements to 0
#sets the elements that corresponds to the x and y values in data to 1
def to_grid_2d(data, factor, shape=(8070, 3780)):
    shape = (int(shape[0]/factor), int(shape[1]/factor))
    grid = torch.zeros(shape)
    for x in data:
        grid[int(x[0])][int(x[1])] = 1.0
    grid = grid.view(1, 1, shape[0], shape[1])
    return grid


#Returns an array with two elements corresponding to the x and y values of the target
#The values will be scaled fram (shape/factor) to target shape
def to_target(data, factor, shape=(8070, 3780), target_shape=(19, 10)):
    shape = (int(shape[0] / factor), int(shape[1] / factor))
    x = (data[0] / shape[0]) * target_shape[0]
    y = (data[1] / shape[1]) * target_shape[1]
    targ = np.empty([2])
    targ[0] = y
    targ[1] = x
    del x, y
    return targ


#Tests the model
def test():
    factor = 4
    shape = (8070, 3780)
    map_nr = 0
    device = "cuda:0" if torch.cuda.is_available() else "cpu"
    x_train, x_test = df.get_data_from_dir("Data/Datasets/set1", 2, factor=factor)
    model = load_model(4, "Data/Models/Conv/conv_linear_2019-04-10 12_44_42.955424.pt", device)

    predictions, target = predict(x_train, factor, model, shape, map_nr)

    for i in range(predictions.shape[0]):
        if i % 100 == 0:
            print(predictions[i])
            print(to_target(target[0][i], factor))
            vis.visualize_heat_interest(predictions[i], to_target(target[0][i], factor))


if __name__ == "__main__":
    test()
