import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import Data.DataFormatter as df
import Data.InterestPoint as ip
import Visualize.Visualize as vis
import numpy as np
import torch

import Predictors.Utility as util
import Models.GridModel as gm


#Loads the trained model with the weights specified by path
#device_string is a string that should be "cuda:0" for gpu and "cpu" for cpu
#input_size should be the size of each row in the input, and hidden_size should be the same as it was trained with
def load_model(input_size, hidden_size, device, path, device_string):
    model = gm.GridModel(input_size, hidden_size, device)
    model.load_state_dict(torch.load(path, map_location=device_string))
    model.eval()
    return model


#creates a grid with the size of shape scaled by factor, and initializes all elements to 0
#sets the elements that corresponds to the x and y values in data to 1
def to_grid_2d(data, shape):
    grid = torch.zeros(shape)
    for i in range(int(len(data)/4)):
        grid[int(data[(i*4)])][int(data[(i*4)+1])] = 1.0

    return grid.view(-1)


#Returns an array with two elements corresponding to the x and y values of the target
#The values will be scaled fram shape to target shape
def to_target(data, shape, target_shape):
    x = (data[0] / shape[0]) * target_shape[0]
    y = (data[1] / shape[1]) * target_shape[1]
    targ = np.empty([2])
    targ[0] = y
    targ[1] = x
    del x, y
    return targ


#Tests the model
def test():
    factor = 100
    shape = (int(8070/factor)+1, int(3780/factor)+1)
    train, test = df.get_data_from_dir("Data/Datasets/set1", 2, factor=factor)
    testern = test[0]
    int_point = ip.InterestPoint([7908/factor, 1890/factor, 830/factor])
    y = int_point.get_most_interesting_positions([testern])
    #target = np.reshape(y[0], [y[0].shape[0], 1, y[0].shape[1]])
    testern = torch.from_numpy(np.reshape(testern, [testern.shape[0], -1])).float()
    device_string = "cuda:0" if torch.cuda.is_available() else "cpu"
    device = torch.device(device_string)
    testern = testern.to(device)

    gridshape = list(to_grid_2d(testern[0], shape).shape)[0]
    model = load_model(gridshape, 400, device, "Data/Models/Grid/grid_tanh_leaky_softmax_ss1_2019-04-29 11_19_52.809258.pt", device_string)

    model.to(device)
    step_size = 1

    predictions = np.empty([testern.shape[0]-step_size, gridshape])

    with torch.no_grad():
        for i in range(predictions.shape[0]):
            pred = model(to_grid_2d(testern[i], shape).to(device))
            pred = pred.view(shape)
            if i % 1000 == 0:
                pred = pred.cpu().numpy()
                print(pred.min())
                print(pred.max())
                vis.visualize_heat_interest(pred, to_target(y[0][i], shape, (81, 38)))


if __name__ == "__main__":
    test()