import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import Data.DataFormatter as df
import Data.InterestPoint as ip
import Visualize.Visualize as vis
import numpy as np
import torch
from datetime import datetime

import Predictors.Utility as util
import Models.Interestv2 as iv2


#Loads the trained model with the weights specified by path
#input_size should be the size of each row in the input
def load_intv2_model(input_size, device, path):
    model = iv2.InterestModel(input_size, input_size, device)
    model.load_state_dict(torch.load(path, map_location='cpu'))
    model.eval()
    return model


#Returns predictions along with their respective targets for each timestep in the match specified by map_nr
#$steps should be the same as the model was trained with
def predict_int_v2(data, steps, match_nr, model):
    with torch.no_grad():
        int_point = ip.InterestPoint([0.98, 0.50, 0.82])
        y = int_point.get_most_interesting_positions_alt(data, steps)
        data = data[match_nr]
        data = util.make_tensor(data)
        predictions = np.empty([data.shape[0], 4])
        for i in range(data.shape[0]):
            predictions[i] = model(data[i])

        return predictions, y[match_nr]


#Tests the model
def test_intv2():
    train, test = df.get_data_from_dir("Data/Datasets/set1", 2, scale=True)
    device = torch.device("cpu")
    model = load_intv2_model(test[0].shape[1] * test[0].shape[2], device, "Data/Models/intv2/leaky_linear_ss1_5k_2019-04-12 12_33_37.724191.pt")

    predictions, target = predict_int_v2(test, 20, 0, model)
    evaluation = util.evaluate(predictions, target)
    predictions = np.reshape(predictions, [predictions.shape[0], 1, predictions.shape[1]])
    target = np.reshape(target, [target.shape[0], 1, target.shape[1]])
    print(predictions.shape)
    print(target.shape)
    print(evaluation)
    vis.visualize_predicted_real((1, 1, 1), predictions, target)


if __name__ == "__main__":
    test_intv2()