import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import Data.DataFormatter as df
import Data.InterestPoint as ip
import Visualize.Visualize as vis
import numpy as np
import torch
from datetime import datetime

import Predictors.Utility as util
import Models.LstmModel1 as lm


#Loads the trained model with the weights specified by path
#input_size should be the size of each row in the input, and hidden_size should be the same as it was trained with
def load_model(input_size, hidden_size, device, path):
    model = lm.LstmModel1(input_size, hidden_size, device)
    model.load_state_dict(torch.load(path, map_location='cpu'))
    model.eval()
    return model


#Returns predictions along with their respective targets for each timestep in the match specified by map_nr, except the first timesteps
#corresponding to step_size
def predict(data, match_nr, model, step_size):
    with torch.no_grad():
        int_point = ip.InterestPoint([0.98, 0.50, 0.82])
        y = int_point.get_most_interesting_positions_old(data)[match_nr]
        data = util.make_tensor(data[match_nr])
        predictions = np.empty([data.shape[0]-step_size, data.shape[1]])
        model.hidden = model.init_hidden()
        for i in range(predictions.shape[0]):
            pred = model(data[i], 1)
            preserve_hidden = model.hidden
            for j in range(step_size-1):
                pred = model(pred, 1)
            predictions[i] = pred
            model.hidden = preserve_hidden
        return predictions, y[step_size:]


#Tests the model on all the matches in the testset
#visualizes each one, prints the evaluation for each, and prints the avarage evaluation of all the matches at the end
def test():
    n_players = 2
    step_size = 20
    train, test = df.get_data_from_dir("Data/Datasets/set1", n_players, scale=True)
    device = torch.device("cpu")
    model = load_model(test[0].shape[1] * test[0].shape[2], 64, device, "Data/Models/Model1/step_size_1.pt")

    ev = 0.0
    for i in range(len(test)):
        predictions, targets = predict(test, i, model, step_size)
        predictions = np.reshape(predictions, [predictions.shape[0], n_players, 4])[:, :1, :3]
        targets = targets[:, :1, :3]

        evaluation = util.evaluate(predictions, targets)
        print(evaluation)
        vis.visualize_predicted_real((1, 1, 1), targets, predictions)
        ev += evaluation
    ev = ev/len(test)
    print("evaluation", ev)


if __name__ == "__main__":
    test()