import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import Data.DataFormatter as df
import Data.InterestPoint as ip
import Visualize.Visualize as vis
import numpy as np
import torch
from datetime import datetime

import Predictors.Utility as util
import Models.LstmModel2 as rm


#Loads the trained model with the weights specified by path
#input_size should be the size of each row in the input, and hidden_size should be the same as it was trained with
def load_model(input_size, hidden_size, device, path):
    model = rm.LstmModel2(input_size, hidden_size, device)
    model.load_state_dict(torch.load(path, map_location='cpu'))
    model.eval()
    return model


#Returns predictions along with their respective targets for each timestep in the match specified by map_nr, except the first timesteps
#corresponding to step_size
def predict(data, map_nr, model, step_size):
    with torch.no_grad():
        int_point = ip.InterestPoint([0.98, 0.50, 0.82])
        y = int_point.get_most_interesting_positions(data)[map_nr]
        data = util.make_tensor(data[map_nr])
        y = util.make_tensor(y)

        predictions = np.empty([data.shape[0] - step_size, 3])
        targets = np.empty([data.shape[0] - step_size, 3])

        for i in range(predictions.shape[0]):
            predictions[i] = model(data[i]).cpu().numpy()
            targets[i] = y[i + step_size].cpu().numpy()

        return predictions, targets


#Tests the model on all the matches in the testset
#visualizes each one, prints the evaluation for each, and prints the avarage evaluation of all the matches at the end
def test():
    train, test = df.get_data_from_dir("Data/Datasets/set1", 2, scale=True)
    device = torch.device("cpu")
    step_size = 1
    hidden = 20
    model = load_model(test[0].shape[1] * test[0].shape[2], hidden, device, "Data/Models/Model2/step_size_20.pt")

    ev = 0.0
    for i in range(len(test)):
        predictions, targets = predict(test, i, model, step_size)
        evaluation = util.evaluate(predictions, targets)
        print(evaluation)
        ev += evaluation

        predictions = np.reshape(predictions, [predictions.shape[0], 1, 3])
        targets = np.reshape(targets, [targets.shape[0], 1, 3])

        vis.visualize_flat((1,1,1), targets, predictions)

    ev = ev / len(test)
    print("evaluation: ", ev)


if __name__ == "__main__":
    test()