import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import Data.DataFormatter as df
import Visualize.Visualize as vis
import numpy as np
import torch

import Predictors.Utility as util
import Models.PositionModel as pm


#Loads the trained model with the weights specified by path
#input_size should be the size of each row in the input
def load_model(input_size, device, path):
    model = pm.PositionModel(input_size, input_size, device)
    model.load_state_dict(torch.load(path, map_location='cpu'))
    model.eval()
    return model


#Returns predictions along with their respective targets for each timestep in the match specified by map_nr, except the first timesteps
#corresponding to step_size
def predict(data, map_nr, model, step_size):
    with torch.no_grad():
        data = util.make_tensor(data[map_nr])
        predictions = np.empty([data.shape[0] - step_size, data.shape[1]])
        targets = np.empty([data.shape[0] - step_size, data.shape[1]])

        for i in range(predictions.shape[0]):
            predictions[i] = model(data[i]).cpu().numpy()
            targets[i] = data[i + step_size].cpu().numpy()

        return predictions, targets


#Tests the model on all the matches in the testset
#visualizes each one, prints the evaluation for each, and prints the avarage evaluation of all the matches at the end
def test():
    train, test = df.get_data_from_dir("Data/Datasets/set1", 2, scale=True)
    step_size = 20
    testern = test[0]
    device = torch.device("cpu")
    n_players = testern.shape[1]
    model = load_model(testern.shape[1] * testern.shape[2], device, "Data/Models/PositionModel/scaler_tanh_leaky_linear_ss202019-03-27 11_33_04.571705.pt")

    ev = 0.0
    for i in range(len(test)):
        prediction, target = predict(test, i, model, step_size)
        evaluation = util.evaluate(prediction, target)
        ev += evaluation
        print("Evaluation: ", evaluation)

        prediction = np.reshape(prediction, [prediction.shape[0], n_players, -1])[:, :, :3]
        target = np.reshape(target, [target.shape[0], n_players, -1])[:, :, :3]
        vis.visualize_predicted_real((1, 1, 1), target, prediction)

    ev = ev/len(test)
    print("Evaluation: ", ev)


if __name__ == "__main__":
    test()