import sys
import os.path
libdir = os.path.dirname(__file__)
sys.path.append(os.path.split(libdir)[0])

import numpy as np
import torch


#Scales the data in coords back to its original shape
#Used after the getting predictions from a model that uses scaled data
#min_shape, scalers and to_scale needs to be the same as it was scaled with
#each element in coords with its index in to_scale is scaled by the corresponding index in scalers
#min shape is then added to the first to elements, to scale back the x and y values that were added to avoid negative values
def inv_scale(coords, min_shape, scalers, to_scale):
    for j in range(len(to_scale)):
        coords[to_scale[j]] = coords[j] * scalers[j]
    coords[0] += min_shape[0]
    coords[1] += min_shape[1]
    return coords


#Uses inv_scale to reverse the scaling of an entire match
def inv_scale_match(match, min_shape=(-2670, -1890), scalers=(8070, 3780, 1000), to_scale=[0, 1, 2]):
    scaled = np.zeros(match.shape)
    for i in range(match.shape[0]):
        for j in range(match.shape[1]):
            scaled[i][j] = inv_scale(match[i][j], min_shape, scalers, to_scale)
    return scaled


#Reshapes data to [timesteps, features] and converts it to a tensor
def make_tensor(data):
    data = np.reshape(data, [data.shape[0], -1])
    return torch.from_numpy(data).float()


#Returns the Mean Square Error of predicted vs target
#Multiplies the result by 100 to get more readable numbers
def evaluate(predicted, target):
    loss = (np.square(target - predicted)).mean(axis=None)
    return loss*100