import matplotlib.pyplot as plt

def visualize_flat(map, real_data, pred_data):
    colors = ['b', 'g', 'r', 'c', 'm', 'y', 'k', 'w', 'pink', 'gray']

    def reshape_data(data):
        """ Reshapes input data to a form which matplotlib accepts
        :param data: the input data in form [[p1_x, p1_y, p1_z], [p2_x, p2_y, p2_z], ...]
        :return: reshaped data as [[[p1_x1, p1_x2, ...], [p1_y1, p1_y2, ...], [p1_z1, p1_z2, ...]], [[p2_x, ...][p2_y, ...][p2_z, ...]], ...]
        """
        num_players = len(data[0])
        reshaped_data = []

        for x in range(0, num_players):
            reshaped_data.append([[],[]])

        for state in data:
            for x in range(0, num_players):
                reshaped_data[x][0].append(state[x][0])
                reshaped_data[x][1].append(state[x][1])
        return reshaped_data

    def show_all(reshaped_real, reshaped_pred):
        """ Visualized all the input data

        :param reshaped_real: the reshaped data for real positions
        :param reshaped_pred: the reshaped data for predicted positions

        """
        
        counter = 0
        for player in reshaped_real:
            plt.plot(player[0], player[1], '-', c=colors[counter], label="real P%i" % (counter+1))
            counter +=1

        counter = 0
        for player in reshaped_pred:
            plt.plot(player[0], player[1], ':', c=colors[counter], label="pred P%i" % (counter+1))
            counter +=1



    plt.xlabel("X-axis")
    plt.ylabel("Y-axis")
    reshaped_real = reshape_data(real_data)
    reshaped_pred = reshape_data(pred_data)
    show_all(reshaped_real, reshaped_pred)
    plt.show()