import numpy as np
import matplotlib as mplt
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.cm as cm
from scipy import array, newaxis

def visualize_heat(data):
    """
    Creates a heatmap visualization

    :param data: 2D list with values that can be representetd in a heatmap
    """

    fig, ax = plt.subplots()
    map = np.zeros((data.shape[0], data.shape[1]))
    for i in range(data.shape[0]):
        for j in range(data.shape[1]):
            map[i, j] = sum(data[i, j])
    im = ax.imshow(map)
    mplt.colors.Normalize(vmin=-1.,vmax=1.)
    plt.show()


def visualize_heat_interest(heat_map, interest):
    """
    Creates a heatmap visualization with plottet points ontop

    :param heat_map: matrix that can be representet in a 2D heatmap
    :param interest: Points of intrest that can be plotted over the heatmap
    """

    fig, ax = plt.subplots()
    im = ax.imshow(heat_map)
    mplt.colors.Normalize(vmin=0., vmax=1.)
    print(interest)
    ax.plot(interest[0], interest[1], color='k', marker='o')
    plt.show()


def visualize_path(map, data):
    """
    Creates a 3D coordinate system where data can be plotted

    :param map: matrix that can be representet in a 3D coordinate system. Used to size coordinate system
    :param data: Collection of states that is used to plot the continous progression of players in the map
    """

    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    ax.set_xlim3d(0,map[0])
    ax.set_ylim3d(0,map[1])
    ax.set_zlim3d(0,map[2])
    color = ['b', 'g', 'r', 'c', 'm', 'y', 'k', 'w']
    #colors = cm.rainbow(np.linspace(0, 1, len(data[0])))
    counter = 0
    for state in data:
        counter = 0
        for player in state:
            ax.scatter(player[0], player[1], player[2], c=color[counter], marker='o')
            counter += 1
    
    plt.show()

def visualize_astar(map, obstacles, path):
    """
    Creates a 3D coordinate system where A* progression can be plotted with obstacles

    :param map: matrix that can be representet in a 3D coordinate system. Used to size coordinate system
    :param obstacles: A list with tuples representing obstables start and end coordinate
    :param path: Collection of positions that is used to plot the continous progression of players in the map
    """

    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    ax.set_xlim3d(0, map.shape[0])
    ax.set_ylim3d(0, map.shape[1])
    ax.set_zlim3d(0,map.shape[2])
    
    for obstacle in obstacles:
        X = np.linspace(obstacle[0][0], obstacle[1][0], num=100)
        Y = np.linspace(obstacle[0][1], obstacle[1][1], num=100)
        x = []
        y = []
        z = []
        for i in range(100):
            x.append([X[i], X[i]])
            y.append([Y[i], Y[i]])
            z.append([0, map.shape[2]])

        x = np.array(x)
        y = np.array(y)
        z = np.array(z)

        
        ax.plot_surface(x, y, z)


    color = ['b', 'g', 'r', 'c', 'm', 'y', 'k', 'w']
    counter = 0
    ax.set_xlim3d(0, map.shape[0])
    ax.set_ylim3d(0, map.shape[1])
    ax.set_zlim3d(0,map.shape[2])
    for position in path:
        ax.scatter(position[0], position[1], position[2], c='g', marker='o')
        counter += 1

    plt.show()

##
